unit TSimpEditorPanelSrc;

interface
uses
  SysUtils, Windows, Messages, Classes, Graphics,
  typinfo, stdctrls, extctrls,forms,  Spin, comctrls, Controls ;

type
PEditorElement = ^TEditorElement;
TEditorElement = packed record
  labelptr : pointer;
  winctrlptr : pointer;
end;

TSimpleEditorPanel = class(tscrollbox)
 Private
    pw, vw : integer;
   fSplitPos : Integer;
   fControlList : TList;
   fHeightList : TList;
   fDefaultRowHeight : Integer;
   inittop : Integer;
   fPPropInfo : PPropInfo;
   inthchange : boolean;

 Protected
   procedure Panel2Resize(Sender: TObject);
   procedure SelfResize(Sender: TObject);
   Procedure SetSplitPos(ASplitPos : Integer);
   Procedure ComboChange(Sender: TObject);
   Procedure makecaptions;
   function MakeCombo : twincontrol;
   function MakePicker : twincontrol;
   function MakeSpin : twincontrol;
   function MakeEdit : twincontrol;

 Public
        fobject : tobject;
        Splitter : TSplitter;
        HolderPanel : TPanel;
        PropertyCaptionLabel, PropertyValueLabel : Tlabel;
        Constructor Create(aowner : tcomponent); Override;
        Destructor Destroy; Override;
        procedure PlaceControls;
        procedure ShowAllProperties(aobject : tobject);
        procedure GetAllProperties(aobject : tObject);
        procedure RefreshProperties(aobject : tobject);
        procedure freeAllControls;
        function leftwidth : integer;
        Property ControlList : TList Read fControlList Write fControlList;
        Property HeightList : TList Read fHeightList Write fHeightList;

 Published
   Property SplitPos : Integer Read fSplitPos Write SetSplitPos;
   Property DefaultRowHeight : Integer Read fDefaultRowHeight Write fDefaultRowHeight;

end;

procedure Register;

procedure EnumNames2ComboBox(TypeInfo: PTypeInfo; aComboBox : TComboBox; Instr : string);

implementation

uses
  cybmanip;

const
     vspace = 10;
     eesize = sizeof(TEditorElement);
     fscrollbarwidth = 20;
     hspace = 10;
     vgap = 5;
     woffset : integer = 12;
     leftpos = 3;
     toppos = 2;
     dot = '.';

procedure Register;
begin
  RegisterComponents('Cyborg', [TSimpleEditorPanel]);
end;

Constructor TSimpleEditorPanel.Create(aowner : tcomponent);

begin
  inherited create(aowner);
{  fEditorList := TEditorList.create;}
  fobject := nil;
  fdefaultrowheight := 24;
  fControlList := TList.Create;
  fHeightList := TList.Create;
    HolderPanel := TPanel.Create(nil);
    with HolderPanel do begin
      parent := self;
      top := 0;
      align := alright;
      popupmenu := self.PopupMenu;
    end;
    Splitter := TSplitter.Create(nil);
    with Splitter do begin
      parent := self;
      cursor := crVsplit;
      width := 3;
      top := 0;
      align := alright;
    end;
   makecaptions;
    with HolderPanel do begin
      onresize := Panel2Resize;
    end;
    onresize := SelfResize;
    inthchange := false;
end;

Procedure TSimpleEditorPanel.makecaptions;

begin
    PropertyCaptionLabel := Tlabel.Create(nil);
    with PropertyCaptionLabel do begin
      parent := self;
      caption := 'Property';
      top := toppos;
      pw := width;
      font.style := font.style+[fsUnderLine, fsBold];
    end;
    PropertyValueLabel := Tlabel.Create(nil);
    with PropertyValueLabel do begin
      parent := HolderPanel;
      caption := 'Value';
      top := toppos;
      vw := width;
      font.style := font.style+[fsUnderLine, fsBold];
      inittop := PropertyValueLabel.top+PropertyValueLabel.height+(2*vgap);
    end;
end;

Destructor TSimpleEditorPanel.Destroy;

begin
  freeAllControls;
{  fEditorList.free;}
  fControlList.free;
  fHeightList.free;
  PropertyValueLabel.free;
  PropertyCaptionLabel.free;
  HolderPanel.free;
  Splitter.free;
  inherited destroy;
end;

procedure TSimpleEditorPanel.PlaceControls;
Var
   rw, lw, rh, ch, lc, mc : integer;
   aEditorElement : PEditorElement;
   awincontrol : twincontrol;
{  aTADLPropertyEditor : TTADLPropertyEditor;}

begin
   mc := pred(ControlList.count);
   ch := inittop;
   lw := self.width - fSplitPos - woffset;
   rw := fSplitPos - woffset;
   for lc := 0 to mc do begin
     aEditorElement := PEditorElement(ControlList[lc]);
     rh := integer(HeightList[lc]);
     with aEditorElement^ do begin
        awincontrol := twincontrol(labelptr);
        if (awincontrol <> nil) then begin
            awincontrol.top := ch;
            awincontrol.left := leftpos;
            awincontrol.height := rh;
            awincontrol.width := lw;
        end;
        awincontrol := twincontrol(winctrlptr);
        if (awincontrol <> nil) then begin
            awincontrol.top := ch;
            awincontrol.width := rw;
            awincontrol.left := leftpos;
            awincontrol.height := rh;
        end;
     end;
     inc(ch, rh+vgap);
   end;
end;

procedure TSimpleEditorPanel.SelfResize(Sender: TObject);

begin
  if inthchange then exit;
    with HolderPanel do begin
      height := self.height;
    end;
    with Splitter do begin
      height := self.height;
    end;
end;

procedure TSimpleEditorPanel.Panel2Resize(Sender: TObject);

begin
  if inthchange then exit;
  fSplitPos := HolderPanel.width;
  if (fsplitpos < (width div 10)) then fSplitPos := (width div 10); 
  PropertyCaptionLabel.left := (width-fSplitPos-PropertyCaptionLabel.width) div 2;
  PropertyValueLabel.left := (fSplitPos-PropertyValueLabel.width) div 2;
  PlaceControls;
end;

procedure TSimpleEditorPanel.ShowAllProperties(aobject : tobject);
VAR
   ch, rh, lc : integer;
   astr, bstr : string;
   alist : TPropList;
   awincontrol : twincontrol;
{   aTADLPropertyEditor : TTADLPropertyEditor;}
   aLabel : TLabel;
   aEditorElement : PEditorElement;
{   atag : ttag;}

begin
  freeAllControls;
  fobject := aobject;
  if (fobject = nil) or (fobject.ClassInfo = nil) then exit;
  inthchange := true;
  ch := inittop;
  fillchar( alist, sizeof(TPropList), 0);
  GetPropInfos(fobject.ClassInfo, @alist);
  lc := 0;
  fPPropInfo := alist[lc];
  while (fPPropInfo <> nil) do begin
     getmem(aEditorElement, eesize);
     fillchar(aEditorElement^, eesize, 0);
     rh := DefaultRowHeight;
     astr := fPPropInfo^.name;
     if astr <> '' then begin
         aLabel := TLabel.Create(self);
         with alabel do begin
             parent := self;
             left := 2;
             top := ch;
             caption := split_caps(astr);
             aLabel.alignment := taRightJustify;
         end;
         holderpanel.width := fsplitpos;
         if (aLabel.width > (self.width - fsplitpos-hspace)) then begin
            holderpanel.width := (self.width - aLabel.width-hspace);
            aLabel.autosize := true;
         end;
         aEditorElement^.labelptr := alabel;
         bstr := fobject.classname+dot+astr;
         case fPPropInfo^.PropType^.Kind of
               tkInteger : awincontrol := MakeSpin;
           tkEnumeration : awincontrol := MakeCombo;
{           tkfloat :       awincontrol := MakePicker;}
           else awincontrol := MakeEdit;
         end;
         With awincontrol do begin
           parent := holderpanel;
           top := ch;
         end;
         aEditorElement^.winctrlptr := awincontrol;
         ControlList.add(aEditorElement);
         HeightList.add(pointer(rh));
         inc(ch, rh+vgap);
         inc(lc);
         fPPropInfo := alist[lc];
     end
     else fPPropInfo := nil;
  end;
  inthchange := false;
  SelfResize(nil);
end;

procedure TSimpleEditorPanel.GetAllProperties(aobject : tObject);

VAR
   lc : integer;
   astr, bstr : string;
   alist : TPropList;
   adouble : double;
   aDateTimePicker : TDateTimePicker;
{   aTADLPropertyEditor : TTADLPropertyEditor;}
   aEditorElement : PEditorElement;

begin
  if (fobject = nil) or (fobject.ClassInfo = nil) or (fobject<> aobject) then exit;
  fobject := aobject;
  fillchar( alist, sizeof(TPropList), #0);
  GetPropInfos(fobject.ClassInfo, @alist);
  lc := 0;
  fPPropInfo := alist[lc];
  while (fPPropInfo <> nil) do begin
     if (fobject = nil) or (fobject.ClassInfo = nil) then exit;
     astr := fPPropInfo^.name;
     aEditorElement := PEditorElement(ControlList[lc]);
         case fPPropInfo^.PropType^.Kind of
      tkLString, tkString : with tedit(aEditorElement^.winctrlptr) do begin
                               bstr := text;
                               SetStrProp(fobject, fPPropInfo, bstr);
                            end;
           tkInteger :      with tSpinedit(aEditorElement^.winctrlptr) do begin
                               bstr := inttostr(value);
                               if (bstr <> '') then SetOrdProp(fobject, fPPropInfo, strtoint(bstr));
                            end;
           tkEnumeration : with tComboBox(aEditorElement^.winctrlptr) do begin
                               bstr := inttostr(itemindex);
                               if (bstr <> '') then SetOrdProp(fobject, fPPropInfo, strtoint(bstr));
                            end;
           tkfloat :       with tedit(aEditorElement^.winctrlptr) do begin
                               bstr := text;
                               if (bstr <> '') then setfloatprop(fobject, fPPropInfo, strtofloat(bstr));
                            end;
         end;
     inc(lc);
     fPPropInfo := alist[lc];
  end;
end;

procedure TSimpleEditorPanel.RefreshProperties(aobject : tobject);
VAR
   lc : integer;
   astr : string;
   alist : TPropList;
   adouble : double;
   adateTime : TdateTime;
{  aTADLPropertyEditor : TTADLPropertyEditor;}
  aEditorElement : PEditorElement;
  aDateTimePicker : TDateTimePicker;

begin
  if (fobject = nil) or (fobject.ClassInfo = nil) or (fobject<> aobject) then exit;
  fobject := aobject;
  fillchar( alist, sizeof(TPropList), #0);
  GetPropInfos(fobject.ClassInfo, @alist);
  lc := 0;
  fPPropInfo := alist[lc];
  while (fPPropInfo <> nil) do begin
     if (fobject = nil) or (fobject.ClassInfo = nil) then exit;
     astr := fPPropInfo^.name;
     aEditorElement := PEditorElement(ControlList[lc]);
         case fPPropInfo^.PropType^.Kind of
           tkLString, tkString : with tedit(aEditorElement^.winctrlptr ) do begin
                                      text := GetStrProp(fobject, fPPropInfo);
                                 end;
               tkInteger : with tSpinedit(aEditorElement^.winctrlptr) do begin
                                  value := GetOrdProp(fobject, fPPropInfo);
                           end;
           tkEnumeration : with tComboBox(aEditorElement^.winctrlptr) do begin
                             itemindex := GetOrdProp(fobject, fPPropInfo);
                           end;
           tkfloat : with tedit(aEditorElement^.winctrlptr ) do begin
                          adouble := GetFloatProp(fobject, fPPropInfo);
                          text := floattostrf(adouble, ffgeneral, 2,15);
                     end;
         end;
     inc(lc);
     fPPropInfo := alist[lc];
  end;
end;

procedure TSimpleEditorPanel.freeAllControls;
Var
   lc, mc : integer;
   aEditorElement : PEditorElement;
   awincontrol : twincontrol;
{  aTADLPropertyEditor : TTADLPropertyEditor;}

begin
   inthchange := true;
   mc := pred(ControlList.count);
   for lc := 0 to mc do begin
     aEditorElement := PEditorElement(ControlList[lc]);
     if (aEditorElement <> nil) then with aEditorElement^ do begin
        awincontrol := twincontrol(labelptr);
        if (awincontrol <> nil) then awincontrol.free;
        awincontrol := twincontrol(winctrlptr);
        if (awincontrol <> nil) then awincontrol.free;
        freemem(aEditorElement, eesize);
     end;
   end;
   ControlList.clear;
   HeightList.clear;
   inthchange := false;
end;

Procedure TSimpleEditorPanel.SetSplitPos(ASplitPos : Integer);
Begin
   If (ASplitPos <> fSplitPos) then begin
         fSplitPos := ASplitPos;
         HolderPanel.width := fSplitPos;
   end;
End;


procedure EnumNames2ComboBox(TypeInfo: PTypeInfo; aComboBox : TComboBox; Instr : string);
var
  P: ^ShortString;
  T: PTypeData;
   cp, lc, mc : integer;
   astr : string;

begin
  T := GetTypeData(GetTypeData(TypeInfo)^.BaseType^);
  P := @T^.NameList;
  mc := T^.MaxValue;
  cp := -1;
  for lc := 0 to mc do begin
        astr := p^;
        if (comparetext(astr, instr) = 0) then cp := lc;
        if ((comparetext(astr, 'True') = 0) or (comparetext(astr, 'False') = 0)) then begin
          if (comparetext(astr, 'True') = 0) then astr :='On';
          if (comparetext(astr, 'False') = 0) then astr :='Off';
        end
        else begin
          delete(astr, 1,2);
        end;
        aComboBox.items.add(split_caps(astr));
        Inc(Integer(P), Length(P^) + 1);
  end;
  aComboBox.itemindex := cp;
end;

Procedure TSimpleEditorPanel.ComboChange(Sender: TObject);
begin
  GetAllProperties( fobject);
  RefreshProperties( fobject);
end;

function TSimpleEditorPanel.MakeCombo : twincontrol;
VAR
   mc : integer;
   bstr : string;
   aComboBox : TComboBox;

begin
   mc := GetOrdProp(fobject, fPPropInfo);
   bstr := GetEnumName(fPPropInfo^.PropType^, mc);
   aComboBox := TComboBox.Create(self);
   aComboBox.Parent := HolderPanel;
   EnumNames2ComboBox(fPPropInfo^.PropType^, acombobox, bstr);
   aComboBox.OnChange := self.ComboChange;
   result := aComboBox;
end;

function TSimpleEditorPanel.MakePicker : twincontrol;
VAR
   adateTime : TdateTime;
   astr : string;
   adouble : double;
   aDateTimePicker : TDateTimePicker;

begin
   adouble := GetFloatProp(fobject, fPPropInfo);
   adateTime := TdateTime(adouble);
   aDateTimePicker := TDateTimePicker.Create(self);
     aDateTimePicker.Parent := HolderPanel;
     astr := fPPropInfo^.name;
     if (pos('DATE', uppercase(astr)) <> 0) then begin
       aDateTimePicker.kind := dtkDate;
       aDateTimePicker.date := adatetime;
     end;
     if (pos('TIME', uppercase(astr)) <> 0) then begin
       aDateTimePicker.kind := dtkTime;
       aDateTimePicker.time := adatetime;
     end;
   result := aDateTimePicker;
end;

function TSimpleEditorPanel.MakeSpin : twincontrol;
VAR
   aint : integer;
   aSpinEdit : TSpinEdit;

begin
   aint := GetOrdProp(fobject, fPPropInfo);
   aSpinEdit := TSpinEdit.Create(self);
   aSpinEdit.Parent := HolderPanel;
   aSpinEdit.value := aint;
   result := aSpinEdit;
end;


function TSimpleEditorPanel.MakeEdit : twincontrol;
VAR
   astr : string;
   aedit : tedit;

begin
   astr := GetStrProp(fobject, fPPropInfo);
   aedit := TEdit.Create(self);
   aedit.Parent := HolderPanel;
   aedit.text := astr;
   result := aedit;
end;

function TSimpleEditorPanel.leftwidth : integer;

begin
   result := width - fsplitpos;
end;


end.
